@@ -23,6 +23,7 @@
 #include <string.h>
 #include <hildon/hildon.h>
 #include <gtk/gtk.h>
+#include <gconf/gconf-client.h>
 
 #include <libosso-abook/osso-abook-waitable.h>
 #include <libosso-abook/osso-abook-aggregator.h>
@@ -32,6 +33,7 @@
 #include <libosso-abook/osso-abook-presence.h>
 #include <libosso-abook/osso-abook-roster.h>
 #include <libosso-abook/osso-abook-account-manager.h>
+#include <libosso-abook/osso-abook-settings.h>
 
 enum
 {
@@ -65,6 +67,8 @@ G_DEFINE_TYPE(RTComLogModel, rtcom_log_model, GTK_TYPE_LIST_STORE);
 #define MAX_CACHED_PER_QUERY_FIRST 10
 #define MAX_CACHED_PER_QUERY 100
 
+#define AVATAR_IMAGE_BORDER { 0, 0, 0, 0 }
+
 typedef struct _RTComLogModelPrivate RTComLogModelPrivate;
 struct _RTComLogModelPrivate
 {
@@ -111,10 +115,18 @@ struct _RTComLogModelPrivate
 
     OssoABookAccountManager *account_manager;
     OssoABookWaitableClosure *accman_ready_closure;
+    GHashTable *vcard_field_mapping;
 
     /* Whether the API user has populated the model. If not, don't
      * react to any DBus signals. */
     gboolean in_use;
+
+    /* Idle handler for refreshing the view, used for avoid unneccessary
+     * multiple refreshes. */
+    guint refresh_id;
+
+    GConfClient *gconf_client;
+    guint display_order_notify_id;
 };
 
 typedef struct _caching_data caching_data_t;
@@ -320,6 +332,8 @@ _account_avatar_notify_image_callback(
 
     if(avatar)
     {
+        const guint8 border[4] = AVATAR_IMAGE_BORDER;
+
         g_debug("Avatar changed.");
 
         if (!OSSO_ABOOK_IS_CONTACT(data->contact))
@@ -329,11 +343,11 @@ _account_avatar_notify_image_callback(
             return;
         }
 
-        avatar_pixbuf = osso_abook_avatar_get_image_scaled(
+        avatar_pixbuf = osso_abook_avatar_get_image_rounded(
                 avatar,
                 HILDON_ICON_PIXEL_SIZE_FINGER,
                 HILDON_ICON_PIXEL_SIZE_FINGER,
-                FALSE);
+                TRUE, -1, border);
 
         if(avatar_pixbuf)
         {
@@ -355,6 +369,89 @@ _account_avatar_notify_image_callback(
     }
 }
 
+/* Get a VCard field that holds the remote_uid info
+ * on a specific account. */
+static const gchar *
+vcard_field_for_account (RTComLogModel *model, const gchar *local_uid)
+{
+    RTComLogModelPrivate * priv = RTCOM_LOG_MODEL_GET_PRIV(model);
+    McAccount *acc;
+    McProfile *profile;
+    const gchar *id;
+    const gchar *vcard_field;
+
+    vcard_field = g_hash_table_lookup (priv->vcard_field_mapping, local_uid);
+    if (vcard_field)
+        return vcard_field;
+
+    if (!osso_abook_waitable_is_ready (
+        OSSO_ABOOK_WAITABLE (priv->account_manager), NULL))
+      return NULL;
+
+    acc = osso_abook_account_manager_lookup_by_name (priv->account_manager,
+        local_uid);
+    if (!acc)
+        return NULL;
+
+    id = mc_account_compat_get_profile (acc);
+    profile = mc_profile_lookup (id);
+    if (!profile)
+        return NULL;
+
+    vcard_field = mc_profile_get_vcard_field (profile);
+    if (!vcard_field)
+        return NULL;
+
+    g_hash_table_insert (priv->vcard_field_mapping, g_strdup (local_uid),
+        g_strdup (vcard_field));
+    return vcard_field;
+}
+
+static gchar *
+new_discover_abook_contact (RTComLogModel *model, const char *local_uid,
+    const gchar *remote_uid)
+{
+    RTComLogModelPrivate * priv = RTCOM_LOG_MODEL_GET_PRIV(model);
+    const gchar *vcard_field;
+    GList *contacts = NULL;
+    gchar *remote_ebook_uid = NULL;
+
+    g_return_val_if_fail (osso_abook_waitable_is_ready(
+          OSSO_ABOOK_WAITABLE(priv->abook_aggregator), NULL), NULL);
+
+    vcard_field = vcard_field_for_account (model, local_uid);
+    if (!vcard_field)
+        return NULL;
+
+    if (!strcmp (vcard_field, EVC_TEL))
+      {
+        contacts = osso_abook_aggregator_find_contacts_for_phone_number
+            (OSSO_ABOOK_AGGREGATOR (priv->abook_aggregator), remote_uid, FALSE);
+      }
+    else
+      {
+        EBookQuery *query;
+
+        query = e_book_query_vcard_field_test (vcard_field,
+                                               E_BOOK_QUERY_IS,
+                                               remote_uid);
+        contacts = osso_abook_aggregator_find_contacts
+            (OSSO_ABOOK_AGGREGATOR (priv->abook_aggregator), query);
+        e_book_query_unref (query);
+      }
+
+    if (contacts)
+    {
+        remote_ebook_uid = g_strdup (
+            osso_abook_contact_get_persistent_uid (OSSO_ABOOK_CONTACT (contacts->data)));
+        g_list_free (contacts);
+
+        g_debug ("%s: discovered remote contact: %s for remote uid %s on account %s",
+            G_STRFUNC, remote_ebook_uid, remote_uid, local_uid);
+    }
+
+    return remote_ebook_uid;
+}
 
 static gchar *
 discover_abook_contact(
@@ -366,6 +463,8 @@ discover_abook_contact(
     RTComLogModelPrivate * priv = RTCOM_LOG_MODEL_GET_PRIV(model);
     GList *li;
 
+    return new_discover_abook_contact (model, local_uid, remote_uid);
+
     g_return_val_if_fail (osso_abook_waitable_is_ready(
           OSSO_ABOOK_WAITABLE(priv->abook_aggregator), NULL), NULL);
 
@@ -1247,18 +1346,13 @@ _new_event_callback (
     if (!priv->in_use)
         return;
 
-    g_debug("service: %s, filtered_services: %s", service,
-      (priv->filtered_services != NULL) ? "yes" : "no");
     if((service != NULL) && (priv->filtered_services != NULL))
     {
         int i;
 
-        g_debug("Checking if we want to show this at all");
-
         /* Check if we're actually filtering this service */
         for(i = 0; priv->filtered_services[i]; i++)
         {
-            g_debug("Checking '%s' vs '%s'", service, priv->filtered_services[i]);
             if(strcmp(service, priv->filtered_services[i]) == 0)
             {
                 /* Ok, here it is, we can continue to staging */
@@ -1431,11 +1525,8 @@ _event_updated_callback (
         return;
 
     if (!_find_event(model, event_id, local_uid, remote_uid, remote_ebook_uid,
-          group_uid, &iter, &id_iter))
-    {
-        g_warning(G_STRLOC ": no matching event found in the model.");
-        return;
-    }
+        group_uid, &iter, &id_iter))
+      return;
 
     g_debug(G_STRLOC ": row found. Need to update icon and text.");
 
@@ -1479,7 +1570,7 @@ _event_updated_callback (
             icon_name = g_value_get_string(g_value_array_get_nth(values, 0));
             text = g_value_get_string(g_value_array_get_nth(values, 1));
             remote_name = g_value_get_string(g_value_array_get_nth(values, 2));
-            event_count = g_value_get_uint(g_value_array_get_nth(values, 3));
+            event_count = g_value_get_int(g_value_array_get_nth(values, 3));
             group_title = g_value_get_string(g_value_array_get_nth(values, 4));
 
             g_debug("Got icon_name=\"%s\", text=\"%s\", "
@@ -1557,11 +1648,8 @@ _event_deleted_callback (
         return;
 
     if (!_find_event(model, event_id, local_uid, remote_uid, remote_ebook_uid,
-          group_uid, &iter, &id_iter))
-    {
-        g_warning(G_STRLOC ": no matching event found in the model.");
-        return;
-    }
+        group_uid, &iter, &id_iter))
+      return;
 
     switch (priv->group_by)
     {
@@ -2046,18 +2134,38 @@ _all_deleted_callback(
     }
 }
 
+static gboolean
+refresh_idle_cb (gpointer model)
+{
+    RTComLogModelPrivate * priv = RTCOM_LOG_MODEL_GET_PRIV(model);
+
+    rtcom_log_model_refresh(model);
+    priv->refresh_id = 0;
+
+    return FALSE;
+}
+
 static void
 _refresh_hint_callback(
         RTComEl * backend,
         RTComLogModel * model)
 {
     RTComLogModelPrivate * priv = RTCOM_LOG_MODEL_GET_PRIV(model);
-    g_debug("%s: called, refreshing the model", G_STRFUNC);
+    g_debug ("%s: called, refreshing the model", G_STRFUNC);
 
     if (!priv->in_use)
         return;
 
-    rtcom_log_model_refresh(model);
+    /* We provide a 1s delay. If there's lots of updates in bulk,
+     * the next refresh request is likely to come before that times out,
+     * in which case we'll extend the period. The end effect is, we'll
+     * do the update 1s after the last refresh request. */
+    if (priv->refresh_id != 0)
+    {
+        g_source_remove (priv->refresh_id);
+    }
+
+    priv->refresh_id = g_timeout_add_seconds (1, refresh_idle_cb, model);
 }
 
 static void
@@ -2141,6 +2249,41 @@ rtcom_log_model_class_init(
 
 static void _create_abook_account_manager (RTComLogModel *model);
 
+static void
+_name_order_changed_cb (
+    GConfClient *client,
+    guint        notify_id,
+    GConfEntry  *entry,
+    gpointer     data)
+{
+    RTComLogModel *model = RTCOM_LOG_MODEL (data);
+    gboolean valid;
+    GtkTreeIter iter;
+
+    valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter);
+    while (valid)
+    {
+        OssoABookContact *contact;
+
+        gtk_tree_model_get(GTK_TREE_MODEL (model), &iter,
+            RTCOM_LOG_VIEW_COL_CONTACT, &contact, -1);
+
+        if (contact)
+        {
+            const gchar *name =
+                osso_abook_contact_get_display_name (contact);
+
+            if (name != NULL)
+                gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+                    RTCOM_LOG_VIEW_COL_REMOTE_NAME, name, -1);
+
+            g_object_unref (contact);
+        }
+
+        valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &iter);
+    }
+}
+
 static void
 rtcom_log_model_init(
         RTComLogModel * log_model)
@@ -2204,6 +2347,9 @@ rtcom_log_model_init(
     priv->pixbufs_populated = FALSE;
     priv->in_use = FALSE;
 
+    priv->vcard_field_mapping = g_hash_table_new_full (g_str_hash,
+        g_str_equal, g_free, g_free);
+
     priv->new_event_handler = g_signal_connect(
             G_OBJECT(priv->backend),
             "new-event",
@@ -2240,6 +2386,16 @@ rtcom_log_model_init(
 
     _create_abook_account_manager (log_model);
 
+    priv->gconf_client = gconf_client_get_default ();
+    priv->display_order_notify_id = 0;
+
+    if (priv->gconf_client != NULL)
+    {
+        priv->display_order_notify_id = gconf_client_notify_add (
+            priv->gconf_client, OSSO_ABOOK_SETTINGS_KEY_NAME_ORDER,
+            _name_order_changed_cb, log_model, NULL, NULL);
+    }
+
     g_debug("Model initialization completed successfully.");
 }
 
@@ -2251,6 +2407,12 @@ rtcom_log_model_dispose(
 
     _priv_cancel_and_join_threads (priv);
 
+    if (priv->refresh_id)
+    {
+        g_source_remove (priv->refresh_id);
+        priv->refresh_id = 0;
+    }
+
     if(priv->current_query)
     {
         g_debug(G_STRLOC ": unreffing the current query...");
@@ -2304,6 +2466,17 @@ rtcom_log_model_dispose(
     g_object_unref(priv->account_manager);
     priv->account_manager = NULL;
 
+    if (priv->display_order_notify_id)
+        gconf_client_notify_remove (priv->gconf_client,
+            priv->display_order_notify_id);
+    priv->display_order_notify_id = 0;
+
+    if (priv->gconf_client)
+        g_object_unref (priv->gconf_client);
+    priv->gconf_client = NULL;
+
+    g_hash_table_destroy (priv->vcard_field_mapping);
+
     G_OBJECT_CLASS(rtcom_log_model_parent_class)->dispose(obj);
 }
 
