@@ -202,7 +202,7 @@ _key_press_event_cb(
         /* Take care to pass Ctrl+Space on. */
         if (key->keyval == GDK_Return ||
             (key->keyval == GDK_space &&
-             !(key->state & GDK_CONTROL_MASK)))
+             (key->state & GDK_CONTROL_MASK)))
           return FALSE;
 
         if(key->keyval == GDK_BackSpace)
@@ -225,7 +225,7 @@ _key_press_event_cb(
     /* Take care to pass Ctrl+Space on. */
     if(GTK_WIDGET_HAS_FOCUS(priv->entry) &&
         (key->keyval == GDK_space &&
-         !(key->state & GDK_CONTROL_MASK)))
+         (key->state & GDK_CONTROL_MASK)))
       return FALSE;
 
     return gtk_im_context_filter_keypress(priv->im_context, key);;
