@@ -31,6 +31,10 @@
 #include <gconf/gconf-client.h>
 #include <clockcore-public.h>
 
+#include <time.h>
+#include <clockd/libtime.h>
+#include <dbus/dbus.h>
+
 static void
 rtcom_log_view_class_init(
         RTComLogViewClass * klass);
@@ -78,10 +82,14 @@ struct _RTComLogViewPrivate
     struct _cell_data presence_cell;
     struct _cell_data avatar_cell;
 
+    /* For time display settings changed. */
     GConfClient *gconf_client;
     guint time_format_notify_id;
     gboolean use_24h;
 
+    /* For TZ change dbus signals. */
+    DBusConnection *dbus;
+
     GHashTable *text_cell_cache;
     GHashTable *presence_icon_cache;
 
@@ -93,6 +101,10 @@ struct _RTComLogViewPrivate
      * specific information (avatar, group_title)*/
     gboolean show_group_chat;
 
+    /* A flag to control whether to show display names
+     * (default) or uids/phone numbers. */
+    gboolean show_display_names;
+
     /* User-requested fixed width or -1 to adjust
      * width dynamically. */
     gint fixed_width;
@@ -159,46 +171,111 @@ _time_format_changed_cb (
     GConfEntry  *entry,
     gpointer     data)
 {
-    RTComLogViewPrivate * priv = RTCOM_LOG_VIEW_GET_PRIV (data);
-
-    priv->use_24h = gconf_client_get_bool (priv->gconf_client,
+    RTComLogView * view = RTCOM_LOG_VIEW (data);
+    RTComLogViewPrivate * priv = RTCOM_LOG_VIEW_GET_PRIV (view);
+    gboolean new_use_24h = gconf_client_get_bool (priv->gconf_client,
         CLOCK_GCONF_IS_24H_FORMAT, NULL);
 
+    if (new_use_24h == priv->use_24h)
+        return;
+
+    priv->use_24h = new_use_24h;
+
     g_debug ("%s: time format changed to %s, updating", G_STRFUNC,
         priv->use_24h ? "24h clock" : "12h clock");
 
     g_hash_table_remove_all (priv->text_cell_cache);
+    gtk_widget_queue_draw (GTK_WIDGET (view));
+}
+
+static DBusHandlerResult _dbus_tz_change_cb (
+    DBusConnection *conn,
+    DBusMessage *msg,
+    void *data)
+{
+    RTComLogView * view = RTCOM_LOG_VIEW (data);
+    RTComLogViewPrivate * priv = RTCOM_LOG_VIEW_GET_PRIV (view);
+    dbus_int32_t tm;
+
+    /* Check we got the signal we asked for. */
+    if (!dbus_message_is_signal (msg, CLOCKD_INTERFACE, CLOCKD_TIME_CHANGED))
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
+    if (!dbus_message_get_args (msg, NULL, DBUS_TYPE_INT32, &tm,
+        DBUS_TYPE_INVALID))
+      return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
+    /* Time argument is 0 for TZ changes, we're only interested in that  */
+    if (tm != 0)
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
+    g_debug ("%s: called, clearing text cell cache", G_STRFUNC);
+
+    g_hash_table_remove_all (priv->text_cell_cache);
+    gtk_widget_queue_draw (GTK_WIDGET (view));
+
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
 static void
 _init_settings(RTComLogView *log_view)
 {
     GError *err = NULL;
+    DBusError derr;
     RTComLogViewPrivate * priv = RTCOM_LOG_VIEW_GET_PRIV (log_view);
 
     priv->gconf_client = gconf_client_get_default ();
     priv->time_format_notify_id = 0;
     priv->use_24h = TRUE;
 
-    if (!priv->gconf_client)
-        return;
-
-    gconf_client_add_dir (priv->gconf_client, CLOCK_GCONF_PATH,
-        GCONF_CLIENT_PRELOAD_NONE, NULL);
-
-    priv->use_24h = gconf_client_get_bool (priv->gconf_client,
-        CLOCK_GCONF_IS_24H_FORMAT, NULL);
-
-    priv->time_format_notify_id = gconf_client_notify_add (priv->gconf_client,
-        CLOCK_GCONF_IS_24H_FORMAT,
-        _time_format_changed_cb,
-        log_view, NULL, &err);
-
-    if (err)
+    if (priv->gconf_client)
     {
-        g_debug ("%s: error while setting 12/24h gconf notification",
+        gconf_client_add_dir (priv->gconf_client, CLOCK_GCONF_PATH,
+            GCONF_CLIENT_PRELOAD_NONE, NULL);
+
+        priv->use_24h = gconf_client_get_bool (priv->gconf_client,
+            CLOCK_GCONF_IS_24H_FORMAT, NULL);
+
+        priv->time_format_notify_id = gconf_client_notify_add (priv->gconf_client,
+            CLOCK_GCONF_IS_24H_FORMAT,
+            _time_format_changed_cb,
+            log_view, NULL, &err);
+
+        if (err)
+        {
+            g_debug ("%s: error while setting 12/24h gconf notification",
+                G_STRFUNC);
+            priv->time_format_notify_id = 0;
+        }
+    }
+
+    dbus_error_init (&derr);
+    priv->dbus = dbus_bus_get (DBUS_BUS_SYSTEM, &derr);
+    if (dbus_error_is_set (&derr))
+    {
+        g_debug ("%s: can't acquire DBus connection for TZ change listener",
             G_STRFUNC);
-        priv->time_format_notify_id = 0;
+        dbus_error_free (&derr);
+        priv->dbus = NULL;
+    } else {
+        dbus_bool_t ret;
+
+        ret = dbus_connection_add_filter (priv->dbus,
+            (DBusHandleMessageFunction) _dbus_tz_change_cb,
+            (void *) log_view, NULL);
+
+        if (!ret)
+        {
+            g_debug ("%s: can't add dbus filter", G_STRFUNC);
+            dbus_connection_unref (priv->dbus);
+            priv->dbus = NULL;
+        }
+        else
+        {
+            dbus_bus_add_match (priv->dbus,
+                "type='signal',interface='" CLOCKD_INTERFACE
+                "',member='" CLOCKD_TIME_CHANGED "'", NULL);
+        }
     }
 }
 
@@ -215,6 +292,16 @@ _dispose_settings(RTComLogView *log_view)
     if (priv->gconf_client)
         g_object_unref (priv->gconf_client);
     priv->gconf_client = NULL;
+
+    if (priv->dbus)
+    {
+        dbus_connection_remove_filter (priv->dbus,
+            (DBusHandleMessageFunction) _dbus_tz_change_cb,
+            (void *) log_view);
+
+        dbus_connection_unref (priv->dbus);
+        priv->dbus = NULL;
+    }
 }
 
 static void
@@ -252,6 +339,7 @@ rtcom_log_view_init(
     priv->fixed_width                  = -1;
     priv->highlight_new_events         = TRUE;
     priv->show_group_chat              = FALSE;
+    priv->show_display_names           = TRUE;
 
     priv->text_cell_cache = g_hash_table_new_full
         (NULL, NULL, NULL, g_free);
@@ -864,8 +952,9 @@ _text_cell_func(
       is_group_chat (priv, group_uid))
       name_str = group_title;
 
-  if ((name_str == NULL) || (*name_str == '\0'))
-      name_str = remote_name;
+  if (priv->show_display_names &&
+      ((name_str == NULL) || (*name_str == '\0')))
+    name_str = remote_name;
 
   if ((name_str == NULL) || (*name_str == '\0'))
       name_str = remote_uid;
@@ -915,6 +1004,7 @@ _text_cell_func(
   g_free (remote_uid);
   g_free (group_uid);
   g_free (group_title);
+  g_free (count_str);
 
   if (contact != NULL)
       g_object_unref (contact);
@@ -1115,6 +1205,19 @@ rtcom_log_view_set_fixed_width (
     }
 }
 
+void
+rtcom_log_view_set_show_display_names (
+        RTComLogView *view,
+        gboolean show_display_names)
+{
+    RTComLogViewPrivate *priv;
+
+    g_return_if_fail (RTCOM_IS_LOG_VIEW (view));
+    priv = RTCOM_LOG_VIEW_GET_PRIV (view);
+
+    priv->show_display_names = show_display_names;
+    g_hash_table_remove_all (priv->text_cell_cache);
+}
 
 /* vim: set ai et tw=75 ts=4 sw=4: */
 
